<?PHP
/**
 * Guestbook
 * @copyright   CONTREXX CMS - COMVATION AG
 * @author      Comvation Development Team <info@comvation.com>
 * @version     1.0.0
 * @package     contrexx
 * @subpackage  module_guestbook
 * @todo        Edit PHP DocBlocks!
 */

/**
 * @ignore
 */
require_once ASCMS_MODULE_PATH . '/guestbook/Lib.class.php';

/**
 * Guestbook
 *
 * Guestbook frontend
 * @copyright   CONTREXX CMS - COMVATION AG
 * @author      Comvation Development Team <info@comvation.com>
 * @access      public
 * @version     1.0.0
 * @package     contrexx
 * @subpackage  module_guestbook
 */
class Guestbook extends GuestbookLibrary
{
    var $langId;
    var $_objTpl;
    var $statusMessage;
    var $arrSettings = array();


    /**
     * Constructor
     * @param  string   $pageContent
     * @global integer
     * @access public
     */
    function __construct($pageContent)
    {
        global $_LANGID;

        $this->pageContent = $pageContent;
        $this->langId = $_LANGID;

        $this->_objTpl = new HTML_Template_Sigma('.');
        CSRF::add_placeholder($this->_objTpl);
        $this->_objTpl->setErrorHandling(PEAR_ERROR_DIE);

        // get the guestbook settings
        $this->getSettings();
    }


    /**
     * Gets the page
     */
    function getPage()
    {
        if(!isset($_GET['cmd'])){
            $_GET['cmd'] = '';
        }

        switch($_GET['cmd']){
            case 'post':
                $this->_newEntry();
                break;
            default:
                $this->_showList();
                break;
        }
        return $this->_objTpl->get();
    }

    /**
     * Gets the guestbook status
     *
     * @global  ADONewConnection
     * @global  array
     * @global  array
     * @access private
     */

    function _showList()
    {
        global $objDatabase, $_CONFIG, $_ARRAYLANG;

        $this->_objTpl->setTemplate($this->pageContent, true, true);

        // initialize variables
        $i = 1;
        $paging = "";
        $pos = (isset($_GET['pos'])) ? intval($_GET['pos']) : 0;

        /** start paging **/
        $query = "    SELECT         id
                    FROM         ".DBPREFIX."module_guestbook
                    WHERE         ".($this->arrSettings['guestbook_only_lang_entries'] ? "lang_id='$this->langId' AND " : '')."status = 1";
        $objResult = $objDatabase->Execute($query);
        $count = $objResult->RecordCount();
        $paging = getPaging($count, $pos, "&amp;section=guestbook", "<b>".$_ARRAYLANG['TXT_GUESTBOOK_ENTRIES']."</b>", false);
        /** end paging **/

        $this->_objTpl->setVariable("GUESTBOOK_PAGING", $paging);
        $this->_objTpl->setVariable("GUESTBOOK_TOTAL_ENTRIES", $count);

        $query = "    SELECT         id,
								forename,
								name,
                                gender,
                                url,
                                email,
                                comment,
                                ip,
                                location,
                                datetime,
                                UNIX_TIMESTAMP(datetime) AS uTimestamp
                    FROM         ".DBPREFIX."module_guestbook
                    WHERE         ".($this->arrSettings['guestbook_only_lang_entries'] ? "lang_id='$this->langId' AND " : '')."status = 1
                    ORDER BY     id DESC";
        $objResult = $objDatabase->SelectLimit($query, $_CONFIG['corePagingLimit'], $pos);

        while ($objResult !== false and !$objResult->EOF) {
            $class = ($i % 2) ? "row1" : "row2";
            $gender = ($objResult->fields["gender"]=="M") ? $_ARRAYLANG['guestbookGenderMale'] : $_ARRAYLANG['guestbookGenderFemale']; // N/A

            if ($objResult->fields['url']!=""){
                $this->_objTpl->setVariable('GUESTBOOK_URL', '<a href="'.$objResult->fields['url'].'"><img alt="'.$objResult->fields['url'].'" src="'.ASCMS_MODULE_IMAGE_WEB_PATH.'/guestbook/www.gif" style="vertical-align:baseline" border="0" /></a>');
            }
            if ($objResult->fields['email']!=""){
                if ($this->arrSettings['guestbook_replace_at']) {
                    $email = $this->changeMail($objResult->fields['email']);
                } else {
                    $email = $objResult->fields['email'];
                }

                $strMailTo = $this->createAsciiString('mailto:'.$email);
				$strMailAdress = $this->createAsciiString($email);

				$asciiStrGuestbookEmail = '<a href="'.$strMailTo.'"><img alt="'.$strMailAdress.'" src="'.ASCMS_MODULE_IMAGE_WEB_PATH.'/guestbook/email.gif" style="vertical-align:baseline" border="0" /></a>';


                $this->_objTpl->setVariable('GUESTBOOK_EMAIL', $asciiStrGuestbookEmail);

            }

            $this->_objTpl->setVariable(array(
                       'GUESTBOOK_ROWCLASS'   => $class,
					   'GUESTBOOK_FORENAME'	  => htmlentities($objResult->fields["forename"], ENT_QUOTES, CONTREXX_CHARSET),
					   'GUESTBOOK_NAME'	      => htmlentities($objResult->fields["name"], ENT_QUOTES, CONTREXX_CHARSET),
                       'GUESTBOOK_GENDER'      => $gender,
                       'GUESTBOOK_LOCATION'      => htmlentities($objResult->fields["location"], ENT_QUOTES, CONTREXX_CHARSET),
                       'GUESTBOOK_DATE'          => date(ASCMS_DATE_FORMAT, $objResult->fields["uTimestamp"]),
                       'GUESTBOOK_COMMENT'      => nl2br($objResult->fields["comment"]),
                       'GUESTBOOK_ID'          => $objResult->fields["id"],
                       'GUESTBOOK_IP'          => $objResult->fields["ip"]
            ));
            $this->_objTpl->parse('guestbook_row');
            $i++;
            $objResult->MoveNext();
        }
        $this->_objTpl->setVariable("GUESTBOOK_STATUS", $this->statusMessage);
    }


    /**
     * New entry
     *
     * Decides what to do, preview, safe or output the errors
     */
    function _newEntry()
    {
        if (isset($_POST['save'])) {
            if ($this->checkInput()) {
                $this->saveEntry();
            } else {
                $this->_showForm();
            }
        } else {
            $this->_showForm();
        }
    }

    /**
    * shows the submit form
    *
    * @access private
    */

    function _showForm()
    {
        global $_ARRAYLANG;

        $this->_objTpl->setTemplate($this->pageContent);

        $checked = "checked=\"checked\"";

        if (!empty($this->error)) {
            $errors = "<span style=\"color: red\">";
            foreach ($this->error as $error) {
                $errors .= $error . "<br />";
            }
            $errors .= "</span>";

            if ($_POST['malefemale'] == "F") {
                $female_checked = $checked;
                $male_checked = "";
            } else {
                $female_checked = "";
                $male_checked = $checked;
            }

            $this->_objTpl->setVariable(array(
				"FORENAME"			=> htmlentities($_POST['forename'], ENT_QUOTES, CONTREXX_CHARSET),
				"NAME"				=> htmlentities($_POST['name'], ENT_QUOTES, CONTREXX_CHARSET),
                "COMMENT"            => $_POST['comment'],
                "FEMALE_CHECKED"    => $female_checked,
                "MALE_CHECKED"        => $male_checked,
                "LOCATION"            => htmlentities($_POST['location'], ENT_QUOTES, CONTREXX_CHARSET),
                "HOMEPAGE"            => htmlentities($_POST['url'], ENT_QUOTES, CONTREXX_CHARSET),
                "EMAIL"                => htmlentities($_POST['email'], ENT_QUOTES, CONTREXX_CHARSET)
            ));
        }

        require_once ASCMS_LIBRARY_PATH . "/spamprotection/captcha.class.php";
        $captcha = new Captcha();

        $alt = $captcha->getAlt();
        $url = $captcha->getUrl();

        $this->_objTpl->setVariable(array(
            "ERROR"                    => $errors,
            "TXT_CAPTCHA"            => $_ARRAYLANG['txt_captcha'],
            "IMAGE_URL"                => $url,
            "IMAGE_ALT"                => $alt,
            "FEMALE_CHECKED"        => $checked
        ));

        $this->_objTpl->hideBlock('guestbookStatus');
        $this->_objTpl->parse('guestbookForm');
    }

    /**
     * Saves an entry
     */
    function saveEntry()
    {
        global $objDatabase, $_ARRAYLANG;

        $this->_objTpl->setTemplate($this->pageContent);

// TODO: Never used
//        $objValidator = new FWValidator();

		$name 	= $_POST['name'];
		$forename 	= $_POST['forename'];
        $gender = $_POST['malefemale'];
        $mail     = isset($_POST['email']) ? $_POST['email'] : '';
        $comment = $this->addHyperlinking($_POST['comment']);
        $location = $_POST['location'];

        if (strlen($_POST['url']) > 7) {
            $url = $_POST['url'];

            if (!preg_match("%^https?://%", $url)) {
                $url = "http://" . $url;
            }
        }

        $status = $this->arrSettings['guestbook_activate_submitted_entries'];

        $query = "INSERT INTO ".DBPREFIX."module_guestbook
                        (status,
	                     forename,
	                     name,
                         gender,
                         url,
                         datetime,
                         email,
                         comment,
                         ip,
                         location,
                         lang_id)
                 VALUES ($status,
						'".addslashes($name)."',
						'".addslashes($forename)."',
                        '".addslashes($gender)."',
                        '".addslashes($url)."',
                        NOW(),
                        '".addslashes($mail)."',
                        '".addslashes($comment)."',
                        '".addslashes($_SERVER['REMOTE_ADDR'])."',
                        '".addslashes($location)."',
                        ".$this->langId.")";
        $objDatabase->Execute($query);

        if ($this->arrSettings['guestbook_send_notification_email']==1) {
	    	$this->sendNotificationEmail($forename, $name, $comment, $mail);
        }
        $this->statusMessage = $_ARRAYLANG['TXT_GUESTBOOK_RECORD_STORED_SUCCESSFUL']."<br />";
        if ($this->arrSettings['guestbook_activate_submitted_entries'] == 0) {
            $this->statusMessage .= '<b>'.$_ARRAYLANG['TXT_GUESTBOOK_RECORD_STORED_ACTIVATE'].'</b>';
        }

        $this->_objTpl->setVariable(array(
	        "GUESTBOOK_STATUS"       => $this->statusMessage
	    ));

        $this->_objTpl->parse('guestbookStatus');
        $this->_objTpl->hideBlock('guestbookForm');
    }

    /**
     * checks input
     */
    function checkInput()
    {
        global $_ARRAYLANG;

        require_once ASCMS_LIBRARY_PATH . "/spamprotection/captcha.class.php";

        $captcha = new Captcha();
        $objValidator = new FWValidator();

		$_POST['forename'] = strip_tags(contrexx_stripslashes($_POST['forename']));
		$_POST['name'] = strip_tags(contrexx_stripslashes($_POST['name']));
        $_POST['comment'] = htmlentities(strip_tags(contrexx_stripslashes($_POST['comment'])), ENT_QUOTES, CONTREXX_CHARSET);
        $_POST['location'] = strip_tags(contrexx_stripslashes($_POST['location']));
        $_POST['email'] = strip_tags(contrexx_stripslashes($_POST['email']));
        $_POST['url'] = strip_tags(contrexx_stripslashes($_POST['url']));

        if (!$captcha->check($_POST['captcha'])) {
            $this->error[] = $_ARRAYLANG['TXT_CAPTCHA_ERROR'];
        }

		if (empty($_POST['name']) || empty($_POST['forename'])) {
            $this->makeError($_ARRAYLANG['TXT_NAME']);
        }

        if (empty($_POST['comment'])) {
            $this->makeError($_ARRAYLANG['TXT_COMMENT']);
        }

        if (empty($_POST['malefemale'])) {
            $this->makeError($_ARRAYLANG['TXT_SEX']);
        }

        if (empty($_POST['location'])) {
            $this->makeError($_ARRAYLANG['TXT_LOCATION']);
        }

        if (!$objValidator->isEmail($_POST['email'])) {
            $this->makeError($_ARRAYLANG['TXT_EMAIL']);
        }

        if (empty($this->error)) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Makes an error
     */
    function makeError($term)
    {
        global $_ARRAYLANG;

        $this->error[] = $term . " " . $_ARRAYLANG['TXT_IS_INVALID'];
    }

    /**
    * @return void
    * @desc Sends a notification email to the administrator
    */
    function sendNotificationEmail($forename, $name, $comment, $email = null)
    {
        global $_ARRAYLANG, $_CONFIG;

        $message = $_ARRAYLANG['TXT_CHECK_GUESTBOOK_ENTRY']."\n\n";
        $message .= $_ARRAYLANG['TXT_ENTRY_READS']."\n".$forename." ".$name."\n".html_entity_decode($comment, ENT_QUOTES, CONTREXX_CHARSET);
        $mailto = $_CONFIG['coreAdminEmail'];
        $subject = $_ARRAYLANG['TXT_NEW_GUESTBOOK_ENTRY']." ".$_CONFIG['domainUrl'];

        if (@include_once ASCMS_LIBRARY_PATH.'/phpmailer/class.phpmailer.php') {
            $objMail = new phpmailer();

            if ($_CONFIG['coreSmtpServer'] > 0 && @include_once ASCMS_CORE_PATH.'/SmtpSettings.class.php') {
                if (($arrSmtp = SmtpSettings::getSmtpAccount($_CONFIG['coreSmtpServer'])) !== false) {
                    $objMail->IsSMTP();
                    $objMail->Host = $arrSmtp['hostname'];
                    $objMail->Port = $arrSmtp['port'];
                    $objMail->SMTPAuth = true;
                    $objMail->Username = $arrSmtp['username'];
                    $objMail->Password = $arrSmtp['password'];
                }
            }

            $objMail->CharSet = CONTREXX_CHARSET;
            if (isset($email)) {
				$objMail->From = $email;
                $objMail->AddReplyTo($email);
		    } else {
				$objMail->From = $mailto;
            }
            $objMail->Subject = $subject;
            $objMail->IsHTML(false);
            $objMail->Body = $message;
            $objMail->AddAddress($mailto);
            if ($objMail->Send()) {
                return true;
            }
        }

        return false;
    }
}
?>
